@@ -50,6 +50,13 @@ typedef struct Track MarkerTrack;
 MarkerTrack *
 marker_track_default ();
 
+/**
+ * Inits the marker track.
+ */
+void
+marker_track_init (
+  Track * track);
+
 /**
  * Adds a Marker to the Track.\
  *
