@@ -128,14 +128,11 @@ midi_arranger_widget_snap_midi_notes_r (
   int                 dry_run);
 
 /**
- * if midi_notes are within the min border distance grid
- * will auto scroll
+ * Resets the transient of each note in the
+ * arranger.
  */
-
 void
-midi_arranger_widget_auto_scroll(
-	MidiArrangerWidget * self,
-  GtkScrolledWindow *  scrolled_window,
-  int                  transient);
+midi_arranger_widget_reset_transients (
+  MidiArrangerWidget * self);
 
 #endif
