@@ -56,8 +56,7 @@ G_DECLARE_FINAL_TYPE (
  */
 typedef enum PortsExpanderTrackPortType
 {
-  PE_TRACK_PORT_TYPE_PREFADER,
-  PE_TRACK_PORT_TYPE_POSTFADER,
+  PE_TRACK_PORT_TYPE_SENDS,
   PE_TRACK_PORT_TYPE_STEREO_IN,
   PE_TRACK_PORT_TYPE_MIDI_IN,
   PE_TRACK_PORT_TYPE_MIDI_OUT,
