@@ -335,6 +335,15 @@ z_gtk_container_get_single_child (
   g_return_val_if_reached (NULL);
 }
 
+/**
+ * Sets the ellipsize mode of each text cell
+ * renderer in the combo box.
+ */
+void
+z_gtk_combo_box_set_ellipsize_mode (
+  GtkComboBox * self,
+  PangoEllipsizeMode ellipsize);
+
 /**
  * Makes the given notebook foldable.
  *
