@@ -46,7 +46,6 @@
 #include "gui/widgets/center_dock_bot_box.h"
 #include "gui/widgets/clip_editor.h"
 #include "gui/widgets/clip_editor_inner.h"
-#include "gui/widgets/donate_dialog.h"
 #include "gui/widgets/export_dialog.h"
 #include "gui/widgets/main_window.h"
 #include "gui/widgets/midi_arranger.h"
@@ -68,6 +67,7 @@
 #include "utils/flags.h"
 #include "utils/gtk.h"
 #include "utils/io.h"
+#include "utils/localization.h"
 #include "utils/resources.h"
 #include "utils/string.h"
 
@@ -132,11 +132,25 @@ activate_manual (GSimpleAction *action,
                 GVariant      *variant,
                 gpointer       user_data)
 {
-#ifdef _WIN32
-  ShellExecute (
-    0, (LPCSTR)"open",
-    (LPCSTR) "https://manual.zrythm.org",
-    0, 0, SW_SHOWNORMAL);
+#ifdef MANUAL_PATH
+  UiLanguage lang =
+    g_settings_get_enum (
+      S_PREFERENCES,
+      "language");
+  char * lang_code =
+    localization_get_string_code (lang);
+  char * path =
+    g_strdup_printf (
+      "file://%s/%s/index.html",
+      MANUAL_PATH,
+      lang_code);
+  gtk_show_uri_on_window (
+    GTK_WINDOW (MAIN_WINDOW),
+    path,
+    0,
+    NULL);
+  g_free (lang_code);
+  g_free (path);
 #else
   gtk_show_uri_on_window (
     GTK_WINDOW (MAIN_WINDOW),
@@ -151,37 +165,11 @@ activate_chat (GSimpleAction *action,
                 GVariant      *variant,
                 gpointer       user_data)
 {
-#ifdef _WIN32
-  ShellExecute (
-    0, (LPCSTR)"open",
-    (LPCSTR) "https://riot.im/app/#/room/#freenode_#zrythm:matrix.org",
-    0, 0, SW_SHOWNORMAL);
-#else
   gtk_show_uri_on_window (
     GTK_WINDOW (MAIN_WINDOW),
-    "https://riot.im/app/#/room/#freenode_#zrythm:matrix.org",
+    "https://matrix.to/#/#zrythmdaw:matrix.org",
     0,
     NULL);
-#endif
-}
-
-void
-activate_forums (GSimpleAction *action,
-                GVariant      *variant,
-                gpointer       user_data)
-{
-#ifdef _WIN32
-  ShellExecute (
-    0, (LPCSTR)"open",
-    (LPCSTR) "https://forum.zrythm.org",
-    0, 0, SW_SHOWNORMAL);
-#else
-  gtk_show_uri_on_window (
-    GTK_WINDOW (MAIN_WINDOW),
-    "https://forum.zrythm.org",
-    0,
-    NULL);
-#endif
 }
 
 void
@@ -189,13 +177,11 @@ activate_donate (GSimpleAction *action,
                 GVariant      *variant,
                 gpointer       user_data)
 {
-  DonateDialogWidget * donate =
-    donate_dialog_widget_new ();
-  gtk_window_set_transient_for (
-    GTK_WINDOW (donate),
-    GTK_WINDOW (MAIN_WINDOW));
-  gtk_dialog_run (GTK_DIALOG (donate));
-  gtk_widget_destroy (GTK_WIDGET (donate));
+  gtk_show_uri_on_window (
+    GTK_WINDOW (MAIN_WINDOW),
+    "https://liberapay.com/Zrythm",
+    0,
+    NULL);
 }
 
 void
@@ -250,6 +236,8 @@ activate_about (GSimpleAction *action,
       "Waui <wau-wau@tutanota.de>\n"
       "Allan Nordhøy <epost@anotheragency.no>\n"
       "WaldiS <admin@sto.ugu.pl>";
+  char * version =
+    zrythm_get_version (1);
 
   gtk_show_about_dialog (
     GTK_WINDOW (MAIN_WINDOW),
@@ -260,14 +248,16 @@ activate_about (GSimpleAction *action,
     /*"logo-icon-name", "z",*/
     "logo", gtk_image_get_pixbuf (img),
     "program-name", "Zrythm",
-    "comments", "An highly automated, intuitive, Digital Audio Workstation (DAW)",
+    "comments", _("a highly automated and intuitive digital audio workstation"),
     "license-type", GTK_LICENSE_AGPL_3_0,
     "translator-credits", translators,
     "website", "https://www.zrythm.org",
-    "website-label", "Home Page",
-    "version", "v" PACKAGE_VERSION,
+    "website-label", _("Website"),
+    "version", version,
     NULL);
   gtk_widget_destroy (GTK_WIDGET (img));
+
+  g_free (version);
 }
 
 void
@@ -990,13 +980,13 @@ activate_create_midi_track (
 }
 
 void
-activate_create_bus_track (GSimpleAction *action,
+activate_create_audio_bus_track (GSimpleAction *action,
                   GVariant      *variant,
                   gpointer       user_data)
 {
   UndoableAction * ua =
     create_tracks_action_new (
-      TRACK_TYPE_BUS,
+      TRACK_TYPE_AUDIO_BUS,
       NULL,
       NULL,
       TRACKLIST->num_tracks,
@@ -1006,14 +996,14 @@ activate_create_bus_track (GSimpleAction *action,
 }
 
 void
-activate_create_group_track (
+activate_create_audio_group_track (
   GSimpleAction *action,
   GVariant      *variant,
   gpointer       user_data)
 {
   UndoableAction * ua =
     create_tracks_action_new (
-      TRACK_TYPE_GROUP,
+      TRACK_TYPE_AUDIO_GROUP,
       NULL,
       NULL,
       TRACKLIST->num_tracks,
@@ -1068,6 +1058,18 @@ activate_delete_selected_tracks (
     UNDO_MANAGER, ua);
 }
 
+void
+activate_hide_selected_tracks (
+  GSimpleAction *action,
+  GVariant      *variant,
+  gpointer       user_data)
+{
+  g_message ("hiding selected tracks");
+
+  tracklist_selections_toggle_visibility (
+    TRACKLIST_SELECTIONS);
+}
+
 void
 change_state_dim_output (
   GSimpleAction * action,
