@@ -226,7 +226,9 @@ exporter_export (ExportSettings * info)
           engine_process_prepare (
             AUDIO_ENGINE,
             AUDIO_ENGINE->nframes);
-          router_start_cycle (&MIXER->router);
+          router_start_cycle (
+            &MIXER->router, AUDIO_ENGINE->nframes,
+            0, PLAYHEAD);
           engine_post_process (AUDIO_ENGINE);
 
           /* by this time, the Master channel should have its Stereo Out ports filled.
