@@ -30,8 +30,8 @@
 #include "audio/engine_pa.h"
 #endif
 #include "audio/master_track.h"
-#include "audio/instrument_track.h"
 #include "audio/midi.h"
+#include "audio/midi_track.h"
 #include "audio/modulator.h"
 #include "audio/pan.h"
 #include "audio/passthrough_processor.h"
@@ -152,7 +152,7 @@ get_node_name (
       break;
     case ROUTE_NODE_TYPE_PORT:
       return
-        g_strdup (node->port->identifier.label);
+        port_get_full_designation (node->port);
       break;
     case ROUTE_NODE_TYPE_FADER:
       return
@@ -166,6 +166,10 @@ get_node_name (
           "%s Pre-Fader",
           node->prefader->channel->track->name);
       break;
+    case ROUTE_NODE_TYPE_MONITOR_FADER:
+      return
+        g_strdup ("Monitor Fader");
+      break;
     case ROUTE_NODE_TYPE_SAMPLE_PROCESSOR:
       return
         g_strdup ("Sample Processor");
@@ -300,6 +304,12 @@ node_process (
       fader_process (
         node->fader, local_offset, nframes);
     }
+  else if (node->type ==
+             ROUTE_NODE_TYPE_MONITOR_FADER)
+    {
+      fader_process (
+        node->fader, local_offset, nframes);
+    }
   else if (node->type == ROUTE_NODE_TYPE_PREFADER)
     {
       passthrough_processor_process (
@@ -326,7 +336,9 @@ node_process (
           chan = port->track->channel;
 
           if (chan->track->type ==
-                TRACK_TYPE_INSTRUMENT)
+                TRACK_TYPE_INSTRUMENT ||
+              chan->track->type ==
+                TRACK_TYPE_MIDI)
             {
               /* panic MIDI if necessary */
               if (g_atomic_int_get (
@@ -348,8 +360,8 @@ node_process (
                   /* fill midi events to pass to
                    * ins plugin */
                   /*g_message ("filling PR events");*/
-                  instrument_track_fill_midi_events (
-                    (InstrumentTrack *)chan->track,
+                  midi_track_fill_midi_events (
+                    chan->track,
                     g_start_frames,
                     local_offset,
                     nframes,
@@ -403,8 +415,8 @@ node_process (
                   chan->stereo_in);
               break;
             case TRACK_TYPE_MASTER:
-            case TRACK_TYPE_BUS:
-            case TRACK_TYPE_GROUP:
+            case TRACK_TYPE_AUDIO_BUS:
+            case TRACK_TYPE_AUDIO_GROUP:
             case TRACK_TYPE_INSTRUMENT:
             case TRACK_TYPE_CHORD:
             case TRACK_TYPE_MARKER:
@@ -442,145 +454,6 @@ node_process (
             }
         }
 
-      /* if JACK stereo out */
-      else if (
-          port->identifier.owner_type ==
-            PORT_OWNER_TYPE_BACKEND &&
-          port->identifier.type == TYPE_AUDIO &&
-          port->identifier.flow == FLOW_OUTPUT)
-        {
-          if (!AUDIO_ENGINE->exporting)
-            {
-              float * out;
-              int ret;
-              switch (AUDIO_ENGINE->audio_backend)
-                {
-                case AUDIO_BACKEND_JACK:
-#ifdef HAVE_JACK
-                  out =
-                    (float *)
-                    jack_port_get_buffer (
-                      JACK_PORT_T (port->data),
-                      AUDIO_ENGINE->nframes);
-
-                  /* by this time, the Master
-                   * channel should have its
-                   * Stereo Out ports filled.
-                   * pass their buffers to JACK's
-                   * buffers */
-                  for (i = 0; i < port->num_srcs;
-                       i++)
-                    {
-                      for (j = local_offset;
-                           j < local_offset +
-                             nframes;
-                           j++)
-                        {
-                          out[j] +=
-                            port->srcs[i]->buf[j];
-                        }
-                    }
-
-                  /* avoid unused warnings */
-                  (void) out;
-#endif
-
-                  break;
-                case AUDIO_BACKEND_ALSA:
-#ifdef __linux__
-                  /* write interleaved */
-                  for (j = 0; j < port->num_srcs;
-                       j++)
-                    {
-                      if (port ==
-                            AUDIO_ENGINE->
-                              stereo_out->l)
-                      for (i = local_offset;
-                           i < local_offset +
-                             nframes;
-                           i++)
-                          {
-                            AUDIO_ENGINE->
-                              alsa_out_buf[i * 2] +=
-                                port->
-                                  srcs[j]->buf[i];
-                          }
-                      else if (port ==
-                            AUDIO_ENGINE->
-                              stereo_out->r)
-                      for (i = local_offset;
-                           i < local_offset +
-                             nframes;
-                           i++)
-                          {
-                            AUDIO_ENGINE->
-                              alsa_out_buf[
-                                i * 2 + 1] +=
-                              port->srcs[j]->buf[i];
-                          }
-                    }
-#endif
-                  break;
-                case AUDIO_BACKEND_PORT_AUDIO:
-#ifdef HAVE_PORT_AUDIO
-                  if (g_atomic_int_get (
-                    &AUDIO_ENGINE->
-                      filled_stereo_out_bufs))
-                    break;
-                  ret =
-                    g_atomic_int_compare_and_exchange (
-                      &AUDIO_ENGINE->
-                        filled_stereo_out_bufs,
-                      0, 1);
-                  if (ret)
-                    engine_pa_fill_stereo_out_buffs (
-                      AUDIO_ENGINE);
-#endif
-                  break;
-                case AUDIO_BACKEND_DUMMY:
-                  break;
-                default:
-                  break;
-                }
-            }
-        }
-
-      /* if JACK MIDI out */
-      else if (port->identifier.owner_type ==
-               PORT_OWNER_TYPE_BACKEND &&
-          port->identifier.type == TYPE_EVENT &&
-          port->identifier.flow == FLOW_OUTPUT)
-        {
-          if (!AUDIO_ENGINE->exporting)
-            {
-              /*float * out;*/
-              /*int ret;*/
-              switch (AUDIO_ENGINE->audio_backend)
-                {
-                case AUDIO_BACKEND_JACK:
-#ifdef HAVE_JACK
-                  /* by this time, the Master
-                   * channel should have its
-                   * MIDI out port filled.
-                   * pass its buffer to JACK */
-                  port_sum_signal_from_inputs (
-                    port, local_offset,
-                    nframes, noroll);
-
-                  midi_events_copy_to_jack (
-                    port->midi_events,
-                    jack_port_get_buffer (
-                      JACK_PORT_T (port->data),
-                      AUDIO_ENGINE->nframes));
-#endif
-                  break;
-                default:
-                  /* TODO */
-                  break;
-                }
-            }
-        }
-
       else
         {
           port_sum_signal_from_inputs (
@@ -984,6 +857,23 @@ find_node_from_sample_processor (
   return NULL;
 }
 
+static GraphNode *
+find_node_from_monitor_fader (
+  Graph * graph,
+  Fader * fader)
+{
+  GraphNode * node;
+  for (int i = 0; i < graph->n_graph_nodes; i++)
+    {
+      node = graph->graph_nodes[i];
+      if (node->type ==
+            ROUTE_NODE_TYPE_MONITOR_FADER &&
+          node->fader == fader)
+        return node;
+    }
+  return NULL;
+}
+
 static inline GraphNode *
 graph_node_new (
   Graph * graph,
@@ -1001,6 +891,8 @@ graph_node_new (
     node->port = (Port *) data;
   else if (type == ROUTE_NODE_TYPE_FADER)
     node->fader = (Fader *) data;
+  else if (type == ROUTE_NODE_TYPE_MONITOR_FADER)
+    node->fader = (Fader *) data;
   else if (type == ROUTE_NODE_TYPE_PREFADER)
     node->prefader = (PassthroughProcessor *) data;
   else if (type ==
@@ -1301,6 +1193,7 @@ add_port (
            port->num_srcs == 0 &&
            owner != PORT_OWNER_TYPE_PLUGIN &&
            owner != PORT_OWNER_TYPE_FADER &&
+           owner != PORT_OWNER_TYPE_MONITOR_FADER &&
            owner != PORT_OWNER_TYPE_PREFADER)
     {
     }
@@ -1309,6 +1202,8 @@ add_port (
         port->identifier.flow == FLOW_OUTPUT) &&
       !(owner == PORT_OWNER_TYPE_FADER &&
         port->identifier.flow == FLOW_OUTPUT) &&
+      !(owner == PORT_OWNER_TYPE_MONITOR_FADER &&
+        port->identifier.flow == FLOW_OUTPUT) &&
       !(owner == PORT_OWNER_TYPE_PREFADER &&
         port->identifier.flow == FLOW_OUTPUT) &&
       !(owner ==
@@ -1324,6 +1219,9 @@ add_port (
            !(owner == PORT_OWNER_TYPE_FADER &&
              port->identifier.flow ==
                FLOW_INPUT) &&
+           !(owner == PORT_OWNER_TYPE_MONITOR_FADER &&
+             port->identifier.flow ==
+               FLOW_INPUT) &&
            !(owner == PORT_OWNER_TYPE_PREFADER &&
              port->identifier.flow ==
                FLOW_INPUT) &&
@@ -1518,6 +1416,11 @@ graph_new (
     self, ROUTE_NODE_TYPE_SAMPLE_PROCESSOR,
     SAMPLE_PROCESSOR);
 
+  /* add the monitor fader */
+  graph_add_node ( \
+    self, ROUTE_NODE_TYPE_MONITOR_FADER,
+    MONITOR_FADER);
+
   /* add plugins */
   Track * tr;
   Plugin * pl;
@@ -1612,6 +1515,27 @@ graph_new (
     find_node_from_port (self, port);
   node_connect (node, node2);
 
+  /* connect the monitor fader */
+  node =
+    find_node_from_monitor_fader (
+      self, MONITOR_FADER);
+  port = MONITOR_FADER->stereo_in->l;
+  node2 =
+    find_node_from_port (self, port);
+  node_connect (node2, node);
+  port = MONITOR_FADER->stereo_in->r;
+  node2 =
+    find_node_from_port (self, port);
+  node_connect (node2, node);
+  port = MONITOR_FADER->stereo_out->l;
+  node2 =
+    find_node_from_port (self, port);
+  node_connect (node, node2);
+  port = MONITOR_FADER->stereo_out->r;
+  node2 =
+    find_node_from_port (self, port);
+  node_connect (node, node2);
+
   Fader * fader;
   PassthroughProcessor * prefader;
   for (int i = 0; i < TRACKLIST->num_tracks; i++)
@@ -1627,43 +1551,74 @@ graph_new (
       node =
         find_node_from_fader (
           self, fader);
-      port = fader->stereo_in->l;
-      node2 =
-        find_node_from_port (self, port);
-      node_connect (node2, node);
-      port = fader->stereo_in->r;
-      node2 =
-        find_node_from_port (self, port);
-      node_connect (node2, node);
-      port = fader->stereo_out->l;
-      node2 =
-        find_node_from_port (self, port);
-      node_connect (node, node2);
-      port = fader->stereo_out->r;
-      node2 =
-        find_node_from_port (self, port);
-      node_connect (node, node2);
+      if (fader->type == FADER_TYPE_AUDIO_CHANNEL)
+        {
+          port = fader->stereo_in->l;
+          node2 =
+            find_node_from_port (self, port);
+          node_connect (node2, node);
+          port = fader->stereo_in->r;
+          node2 =
+            find_node_from_port (self, port);
+          node_connect (node2, node);
+          port = fader->stereo_out->l;
+          node2 =
+            find_node_from_port (self, port);
+          node_connect (node, node2);
+          port = fader->stereo_out->r;
+          node2 =
+            find_node_from_port (self, port);
+          node_connect (node, node2);
+        }
+      else if (fader->type ==
+                 FADER_TYPE_MIDI_CHANNEL)
+        {
+          port = fader->midi_in;
+          node2 =
+            find_node_from_port (self, port);
+          node_connect (node2, node);
+          port = fader->midi_out;
+          node2 =
+            find_node_from_port (self, port);
+          node_connect (node, node2);
+        }
 
       /* connect the prefader */
       node =
         find_node_from_prefader (
           self, prefader);
-      port = prefader->stereo_in->l;
-      node2 =
-        find_node_from_port (self, port);
-      node_connect (node2, node);
-      port = prefader->stereo_in->r;
-      node2 =
-        find_node_from_port (self, port);
-      node_connect (node2, node);
-      port = prefader->stereo_out->l;
-      node2 =
-        find_node_from_port (self, port);
-      node_connect (node, node2);
-      port = prefader->stereo_out->r;
-      node2 =
-        find_node_from_port (self, port);
-      node_connect (node, node2);
+      if (prefader->type ==
+            PP_TYPE_AUDIO_CHANNEL)
+        {
+          port = prefader->stereo_in->l;
+          node2 =
+            find_node_from_port (self, port);
+          node_connect (node2, node);
+          port = prefader->stereo_in->r;
+          node2 =
+            find_node_from_port (self, port);
+          node_connect (node2, node);
+          port = prefader->stereo_out->l;
+          node2 =
+            find_node_from_port (self, port);
+          node_connect (node, node2);
+          port = prefader->stereo_out->r;
+          node2 =
+            find_node_from_port (self, port);
+          node_connect (node, node2);
+        }
+      else if (prefader->type ==
+                 PP_TYPE_MIDI_CHANNEL)
+        {
+          port = prefader->midi_in;
+          node2 =
+            find_node_from_port (self, port);
+          node_connect (node2, node);
+          port = prefader->midi_out;
+          node2 =
+            find_node_from_port (self, port);
+          node_connect (node, node2);
+        }
 
 #define CONNECT_PLUGIN \
           if (!pl || pl->deleting) \
