@@ -70,6 +70,22 @@ track_lane_new (
   return self;
 }
 
+/**
+ * Updates the frames of each position in each child
+ * of the track lane recursively.
+ */
+void
+track_lane_update_frames (
+  TrackLane * self)
+{
+  Region * r;
+  for (int i = 0; i < self->num_regions; i++)
+    {
+      r = self->regions[i];
+      region_update_frames (r);
+    }
+}
+
 /**
  * Adds a Region to the given TrackLane.
  */
@@ -135,8 +151,11 @@ void
 track_lane_free (
   TrackLane * self)
 {
-  g_object_unref (
-    G_OBJECT (self->widget));
+  if (self->widget)
+    {
+      g_object_unref (
+        G_OBJECT (self->widget));
+    }
 
   if (self->name)
     g_free (self->name);
