@@ -47,6 +47,7 @@
 #include "gui/widgets/chord_arranger.h"
 #include "gui/widgets/chord_editor_space.h"
 #include "gui/widgets/color_area.h"
+#include "gui/widgets/foldable_notebook.h"
 #include "gui/widgets/header_notebook.h"
 #include "gui/widgets/home_toolbar.h"
 #include "gui/widgets/left_dock_edge.h"
@@ -70,7 +71,9 @@
 #include "gui/widgets/toolbox.h"
 #include "gui/widgets/top_bar.h"
 #include "gui/widgets/track.h"
+#include "gui/widgets/track_visibility_tree.h"
 #include "gui/widgets/tracklist.h"
+#include "gui/widgets/visibility.h"
 #include "project.h"
 #include "utils/arrays.h"
 #include "utils/gtk.h"
@@ -222,6 +225,8 @@ on_automation_track_added (AutomationTrack * at)
 
   timeline_arranger_widget_refresh_children (
     MW_TIMELINE);
+
+  visibility_widget_refresh (MW_VISIBILITY);
 }
 
 static void
@@ -531,6 +536,7 @@ on_track_name_changed (Track * track)
   mixer_widget_soft_refresh (MW_MIXER);
   track_widget_refresh (track->widget);
   inspector_widget_refresh (MW_INSPECTOR);
+  visibility_widget_refresh (MW_VISIBILITY);
 }
 
 static void
@@ -666,6 +672,8 @@ events_process (void * data)
           if (MW_TRACKLIST)
             tracklist_widget_hard_refresh (
               MW_TRACKLIST);
+          visibility_widget_refresh (
+            MW_VISIBILITY);
           break;
         case ET_CHANNEL_REMOVED:
           mixer_widget_hard_refresh (
@@ -857,6 +865,8 @@ events_process (void * data)
           if (MW_TRACKLIST)
             tracklist_widget_hard_refresh (
               MW_TRACKLIST);
+          visibility_widget_refresh (
+            MW_VISIBILITY);
           break;
         case ET_TRACK_COLOR_CHANGED:
           on_track_color_changed ((Track *) ev->arg);
@@ -881,6 +891,16 @@ events_process (void * data)
           on_track_state_changed (
             (Track *) ev->arg);
           break;
+        case ET_TRACK_VISIBILITY_CHANGED:
+          tracklist_widget_soft_refresh (
+            MW_TRACKLIST);
+          timeline_arranger_widget_refresh_children (
+            MW_TIMELINE);
+          timeline_arranger_widget_refresh_children (
+            MW_PINNED_TIMELINE);
+          track_visibility_tree_widget_refresh (
+            MW_TRACK_VISIBILITY_TREE);
+          break;
         case ET_UNDO_REDO_ACTION_DONE:
           home_toolbar_widget_refresh_undo_redo_buttons (
             MW_HOME_TOOLBAR);
@@ -947,6 +967,12 @@ events_process (void * data)
           if (MW_TRACKLIST)
             tracklist_widget_hard_refresh (
               MW_TRACKLIST);
+          if (MW_PINNED_TRACKLIST)
+            pinned_tracklist_widget_hard_refresh (
+              MW_PINNED_TRACKLIST);
+
+          visibility_widget_refresh (
+            MW_VISIBILITY);
 
           /* needs to be called later because tracks
            * need time to get allocated */
