@@ -298,6 +298,25 @@ midi_arranger_selections_remove_midi_note (
     note);
 }
 
+/**
+ * Resets the given counterparts from the other
+ * counterparts.
+ *
+ * @param reset_trans 1 to reset the transient from
+ *   main, 0 to reset main from transient.
+ */
+void
+midi_arranger_selections_reset_counterparts (
+  MidiArrangerSelections * self,
+  int                  reset_trans)
+{
+  for (int i = 0; i < self->num_midi_notes; i++)
+    {
+      midi_note_reset_counterpart (
+        self->midi_notes[i], reset_trans);
+    }
+}
+
 /**
  * Clone the struct for copying, undoing, etc.
  */
@@ -461,6 +480,10 @@ midi_arranger_selections_add_ticks (
       mn = mas->midi_notes[i];
       midi_note_move (
         mn, ticks, use_cached_pos, update_flag);
+      /*g_message (*/
+        /*"moving %ld",*/
+        /*ticks);*/
+      /*position_print_simple (&mn->start_pos);*/
     }
 }
 
