@@ -33,7 +33,7 @@
 #include "audio/automation_track.h"
 #include "audio/automation_tracklist.h"
 #include "audio/audio_track.h"
-#include "audio/bus_track.h"
+#include "audio/audio_bus_track.h"
 #include "audio/channel.h"
 #include "audio/chord_object.h"
 #include "audio/chord_track.h"
@@ -393,8 +393,8 @@ automation_arranger_widget_select_all (
   for (i = 0; i < region->num_aps; i++)
     {
       ap = region->aps[i];
-      automation_point_widget_select (
-        ap->widget, select);
+      automation_point_select (
+        ap, select);
     }
 
   /**
