@@ -23,7 +23,7 @@
 
 #include "audio/automation_region.h"
 #include "audio/automation_track.h"
-#include "audio/bus_track.h"
+#include "audio/audio_bus_track.h"
 #include "audio/channel.h"
 #include "audio/instrument_track.h"
 #include "audio/track.h"
