@@ -75,6 +75,15 @@ channel_widget_update_meter_reading (
   double prev = widget->meter_reading_val;
   Channel * channel = widget->channel;
 
+  /* TODO */
+  if (channel->track->out_signal_type ==
+        TYPE_EVENT)
+    {
+      gtk_label_set_text (
+        widget->meter_reading, "-∞");
+      return FALSE;
+    }
+
   /* calc decibels */
   channel_set_current_l_db (
     channel,
@@ -326,7 +335,7 @@ on_drag_motion (
   do_highlight (self, x, y);
 }
 
-void
+static void
 on_drag_leave (GtkWidget      *widget,
                GdkDragContext *context,
                guint           time,
@@ -577,24 +586,28 @@ setup_inserts (ChannelWidget * self)
 static void
 setup_channel_icon (ChannelWidget * self)
 {
-  switch (self->channel->type)
+  switch (self->channel->track->type)
     {
-    case CT_MIDI:
+    case TRACK_TYPE_INSTRUMENT:
+    case TRACK_TYPE_MIDI:
       resources_set_image_icon (self->icon,
                                 ICON_TYPE_ZRYTHM,
                                 "instrument.svg");
       break;
-    case CT_AUDIO:
+    case TRACK_TYPE_AUDIO:
       resources_set_image_icon (self->icon,
                                 ICON_TYPE_ZRYTHM,
                                 "audio.svg");
       break;
-    case CT_BUS:
-    case CT_MASTER:
+    case TRACK_TYPE_AUDIO_BUS:
+    case TRACK_TYPE_AUDIO_GROUP:
+    case TRACK_TYPE_MASTER:
       resources_set_image_icon (self->icon,
                                 ICON_TYPE_ZRYTHM,
                                 "bus.svg");
       break;
+    default:
+      break;
     }
 }
 
@@ -885,6 +898,15 @@ channel_widget_init (ChannelWidget * self)
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
+  /* set font sizes */
+  GtkStyleContext * context =
+    gtk_widget_get_style_context (
+      GTK_WIDGET (self->name->label));
+  gtk_style_context_add_class (
+    context, "channel_label");
+  gtk_label_set_max_width_chars (
+    self->name->label, 10);
+
   /* setup inserts */
   GtkWidget * inserts_scroll =
     gtk_scrolled_window_new (
@@ -955,7 +977,6 @@ channel_widget_init (ChannelWidget * self)
   gtk_widget_set_hexpand (
     GTK_WIDGET (self), 0);
 
-  GtkStyleContext * context;
   z_gtk_container_destroy_all_children (
     GTK_CONTAINER (self->record));
   z_gtk_button_set_icon_name (
