@@ -33,6 +33,7 @@
 #include "gui/widgets/channel_slot.h"
 #include "gui/widgets/main_window.h"
 #include "project.h"
+#include "utils/cairo.h"
 #include "utils/gtk.h"
 #include "utils/flags.h"
 #include "utils/ui.h"
@@ -44,6 +45,8 @@ G_DEFINE_TYPE (ChannelSlotWidget,
                channel_slot_widget,
                GTK_TYPE_DRAWING_AREA)
 
+#define ELLIPSIZE_PADDING 2
+
 static void
 on_drag_data_received (
   GtkWidget        *widget,
@@ -133,11 +136,13 @@ on_drag_data_received (
 }
 
 static int
-draw_cb (GtkWidget * widget, cairo_t * cr, void* data)
+draw_cb (
+  GtkWidget * widget,
+  cairo_t * cr,
+  ChannelSlotWidget * self)
 {
   guint width, height;
   GtkStyleContext *context;
-  ChannelSlotWidget * self = (ChannelSlotWidget *) widget;
   context = gtk_widget_get_style_context (widget);
 
   width = gtk_widget_get_allocated_width (widget);
@@ -148,27 +153,31 @@ draw_cb (GtkWidget * widget, cairo_t * cr, void* data)
 
 
   int padding = 2;
-  cairo_text_extents_t te;
-  Plugin * plugin = self->channel->plugins[self->slot_index];
+  Plugin * plugin =
+    self->channel->plugins[self->slot_index];
   if (plugin)
     {
-      GdkRGBA c;
+      GdkRGBA bg, fg;
+      gdk_rgba_parse (&fg, "#000000");
       if (!plugin->enabled)
-        /* matcha */
-        gdk_rgba_parse (&c,
-                        "#2eb398");
+        {
+          /* matcha */
+          gdk_rgba_parse (&bg, "#2eb398");
+        }
       if (plugin->visible)
-        /* bright green */
-        gdk_rgba_parse (&c,
-                        "#1DDD6A");
+        {
+          /* bright green */
+          gdk_rgba_parse (&bg, "#1DDD6A");
+        }
       else
-        /* darkish green */
-        gdk_rgba_parse (&c,
-                        "#1A884c");
+        {
+          /* darkish green */
+          gdk_rgba_parse (&bg, "#1A884c");
+        }
 
       /* fill background */
       cairo_set_source_rgba (
-        cr, c.red, c.green, c.blue, 1.0);
+        cr, bg.red, bg.green, bg.blue, 1.0);
       cairo_move_to (cr, padding, padding);
       cairo_line_to (cr, padding, height - padding);
       cairo_line_to (cr, width - padding, height - padding);
@@ -176,17 +185,33 @@ draw_cb (GtkWidget * widget, cairo_t * cr, void* data)
       cairo_fill(cr);
 
       /* fill text */
-      cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 1.0);
-      cairo_select_font_face (cr, "Arial",
-                              CAIRO_FONT_SLANT_NORMAL,
-                              CAIRO_FONT_WEIGHT_BOLD);
-      cairo_set_font_size (cr, 12.0);
-      cairo_text_extents (cr, plugin->descr->name, &te);
-      cairo_move_to (cr, 20,
-                     te.height / 2 - te.y_bearing);
-      char * text;
-      text = plugin->descr->name;
-      cairo_show_text (cr, text);
+      cairo_set_source_rgba (
+        cr, fg.red, fg.green, fg.blue, 1.0);
+      int w, h;
+      char * font = "Arial Bold 9";
+      char * text = plugin->descr->name;
+      z_cairo_get_text_extents_for_widget_full (
+        widget, text, &w, &h, font,
+        PANGO_ELLIPSIZE_END, ELLIPSIZE_PADDING);
+      z_cairo_draw_text_full (
+        cr, widget, text, width / 2 - w / 2,
+        height / 2 - h / 2, font,
+        PANGO_ELLIPSIZE_END, ELLIPSIZE_PADDING);
+      if (text != plugin->descr->name)
+        g_free (text);
+
+      /* update tooltip */
+      if (!self->pl_name ||
+          !g_strcmp0 (
+            plugin->descr->name, self->pl_name))
+        {
+          if (self->pl_name)
+            g_free (self->pl_name);
+          self->pl_name =
+            g_strdup (plugin->descr->name);
+          gtk_widget_set_tooltip_text (
+            widget, self->pl_name);
+        }
     }
   else
     {
@@ -199,17 +224,26 @@ draw_cb (GtkWidget * widget, cairo_t * cr, void* data)
       cairo_fill(cr);
 
       /* fill text */
-      const char * txt = "empty slot";
       cairo_set_source_rgba (cr, 0.3, 0.3, 0.3, 1.0);
-      cairo_select_font_face (cr, "Arial",
-                              CAIRO_FONT_SLANT_ITALIC,
-                              CAIRO_FONT_WEIGHT_NORMAL);
-      cairo_set_font_size (cr, 12.0);
-      cairo_text_extents (cr, txt, &te);
-      cairo_move_to (cr, 20,
-                     te.height / 2 - te.y_bearing);
-      cairo_show_text (cr, txt);
+      int w, h;
+      char * font = "Arial Italic 9";
+      char * text = _("empty slot");
+      z_cairo_get_text_extents_for_widget_full (
+        widget, text, &w, &h, font,
+        PANGO_ELLIPSIZE_END, ELLIPSIZE_PADDING);
+      z_cairo_draw_text_full (
+        cr, widget, text, width / 2 - w / 2,
+        height / 2 - h / 2, font,
+        PANGO_ELLIPSIZE_END, ELLIPSIZE_PADDING);
 
+      /* update tooltip */
+      if (self->pl_name)
+        {
+          g_free (self->pl_name);
+          self->pl_name = NULL;
+          gtk_widget_set_tooltip_text (
+            widget, _("empty slot"));
+        }
     }
 
   //highlight if grabbed or if mouse is hovering over me
@@ -640,7 +674,8 @@ channel_slot_widget_new (int slot_index,
 }
 
 static void
-channel_slot_widget_init (ChannelSlotWidget * self)
+channel_slot_widget_init (
+  ChannelSlotWidget * self)
 {
   /* make it able to notify */
   gtk_widget_add_events (
@@ -652,6 +687,10 @@ channel_slot_widget_init (ChannelSlotWidget * self)
   gtk_widget_set_size_request (
     GTK_WIDGET (self), -1, 24);
 
+  self->pl_name = NULL;
+  gtk_widget_set_tooltip_text (
+    GTK_WIDGET (self), _("empty slot"));
+
   self->multipress =
     GTK_GESTURE_MULTI_PRESS (
       gtk_gesture_multi_press_new (
