@@ -104,7 +104,7 @@ chord_draw_cb (
     color, &c2);
   cairo_set_source_rgba (
     cr, c2.red, c2.green, c2.blue, 1.0);
-  z_cairo_draw_text (cr, str);
+  z_cairo_draw_text (cr, widget, str);
   g_free (str);
 
  return FALSE;
@@ -130,10 +130,6 @@ on_press (
     }
 }
 
-DEFINE_ARRANGER_OBJECT_WIDGET_SELECT (
-  ChordObject, chord_object,
-  chord_selections, CHORD_SELECTIONS);
-
 static gboolean
 on_motion (
   GtkWidget *      widget,
