@@ -179,7 +179,7 @@ on_drag_data_received (
           if (plugin_descriptor_is_instrument (pd))
             tt = TRACK_TYPE_INSTRUMENT;
           else
-            tt = TRACK_TYPE_BUS;
+            tt = TRACK_TYPE_AUDIO_BUS;
 
           UndoableAction * ua =
             create_tracks_action_new (
@@ -306,26 +306,26 @@ show_context_menu (DragDestBoxWidget * self)
   GtkMenuItem * menu_item;
   menu = gtk_menu_new();
 
-  menu_item =
-    z_gtk_create_menu_item (
-      _("Add _Instrument Track"),
-      NULL,
-      ICON_TYPE_GNOME_BUILDER,
-      NULL,
-      0,
-      "win.create-ins-track");
-  gtk_menu_shell_append (GTK_MENU_SHELL(menu),
-                         GTK_WIDGET (menu_item));
-  menu_item =
-    z_gtk_create_menu_item (
-      _("Add _Audio Track"),
-      NULL,
-      ICON_TYPE_GNOME_BUILDER,
-      NULL,
-      0,
-      "win.create-audio-track");
-  gtk_menu_shell_append (GTK_MENU_SHELL(menu),
-                         GTK_WIDGET (menu_item));
+  /*menu_item =*/
+    /*z_gtk_create_menu_item (*/
+      /*_("Add _Instrument Track"),*/
+      /*NULL,*/
+      /*ICON_TYPE_GNOME_BUILDER,*/
+      /*NULL,*/
+      /*0,*/
+      /*"win.create-ins-track");*/
+  /*gtk_menu_shell_append (GTK_MENU_SHELL(menu),*/
+                         /*GTK_WIDGET (menu_item));*/
+  /*menu_item =*/
+    /*z_gtk_create_menu_item (*/
+      /*_("Add _Audio Track"),*/
+      /*NULL,*/
+      /*ICON_TYPE_GNOME_BUILDER,*/
+      /*NULL,*/
+      /*0,*/
+      /*"win.create-audio-track");*/
+  /*gtk_menu_shell_append (GTK_MENU_SHELL(menu),*/
+                         /*GTK_WIDGET (menu_item));*/
 
   menu_item =
     z_gtk_create_menu_item (
