@@ -24,7 +24,7 @@
 #include "plugins/lv2_plugin.h"
 #include "audio/automation_track.h"
 #include "audio/automation_tracklist.h"
-#include "audio/bus_track.h"
+#include "audio/audio_bus_track.h"
 #include "audio/engine.h"
 #include "audio/instrument_track.h"
 #include "audio/track.h"
@@ -189,9 +189,8 @@ instrument_track_widget_refresh (
 
   instrument_track_widget_refresh_buttons (self);
 
-  gtk_label_set_text (
-    tw_prv->top_grid->name,
-    track->name);
+  track_widget_set_name (
+    Z_TRACK_WIDGET (self), track->name);
 
   AutomationTracklist * automation_tracklist =
     track_get_automation_tracklist (tw_prv->track);
@@ -271,14 +270,14 @@ instrument_track_widget_init (
     z_gtk_toggle_button_new_with_icon (
       "z-format-justify-fill");
   ui_add_widget_tooltip (
-    self->show_automation,
-    _("Show lanes"));
+    self->show_lanes,
+    _("Show track lanes"));
   ui_set_hover_status_bar_signals (
-    self->show_automation,
+    self->show_lanes,
     _("Show Lanes - Shows the track's lanes"));
   self->lock =
     z_gtk_toggle_button_new_with_icon (
-      "object-unlocked");
+      "z-object-unlocked");
   gtk_widget_set_tooltip_text (
     GTK_WIDGET (self->lock),
     _("Lock track"));
