@@ -103,8 +103,9 @@ on_close_notification_clicked (
 void
 main_window_widget_refresh (MainWindowWidget * self)
 {
-  header_notebook_widget_setup (MW_HEADER_NOTEBOOK,
-                           PROJECT->title);
+  header_notebook_widget_setup (
+    MW_HEADER_NOTEBOOK,
+    PROJECT->title);
 
   /* setup center dock */
   center_dock_widget_setup (MW_CENTER_DOCK);
@@ -141,6 +142,12 @@ main_window_widget_refresh (MainWindowWidget * self)
 
   gtk_window_maximize (
     GTK_WINDOW (self));
+
+  /* show track selection info */
+  EVENTS_PUSH (ET_TRACK_CHANGED,
+               TRACKLIST_SELECTIONS->tracks[0]);
+  EVENTS_PUSH (ET_TRACKLIST_SELECTIONS_CHANGED,
+               NULL);
 }
 
 static void
@@ -255,11 +262,11 @@ main_window_widget_init (MainWindowWidget * self)
     { "create-ins-track",
       activate_create_ins_track },
     { "create-bus-track",
-      activate_create_bus_track },
+      activate_create_audio_bus_track },
     { "create-midi-track",
       activate_create_midi_track },
     { "create-group-track",
-      activate_create_group_track },
+      activate_create_audio_group_track },
     { "delete-selected-tracks",
       activate_delete_selected_tracks },
 
@@ -297,6 +304,8 @@ main_window_widget_init (MainWindowWidget * self)
       activate_delete_selected_tracks },
     { "duplicate-selected-tracks",
       activate_duplicate_selected_tracks },
+    { "hide-selected-tracks",
+      activate_hide_selected_tracks },
 
     /* piano roll */
     { "toggle-drum-mode",
