@@ -189,8 +189,8 @@ midi_arranger_widget_select_all (
   for (int i = 0; i < mr->num_midi_notes; i++)
     {
       MidiNote * midi_note = mr->midi_notes[i];
-      midi_note_widget_select (
-        midi_note->widget, select);
+      midi_note_select (
+        midi_note, select);
     }
   /*arranger_widget_refresh(&self->parent_instance);*/
 }
@@ -413,19 +413,66 @@ midi_arranger_widget_show_context_menu (
   gtk_menu_popup_at_pointer (GTK_MENU(menu), NULL);
 }
 
+/**
+ * Resets the transient of each note in the
+ * arranger.
+ *
+ * @param reset_trans 1 to reset the transient from
+ *   main, 0 to reset main from transient.
+ */
+static void
+midi_arranger_widget_reset_counterparts (
+  MidiArrangerWidget * self,
+  int                  reset_trans)
+{
+  GList *children, *iter;
+  children =
+    gtk_container_get_children (
+      GTK_CONTAINER (self));
+  MidiNoteWidget * mnw = NULL;
+  for (iter = children;
+       iter != NULL;
+       iter = g_list_next (iter))
+    {
+      if (!Z_IS_MIDI_NOTE_WIDGET (iter->data))
+        continue;
+
+      mnw =
+        Z_MIDI_NOTE_WIDGET (iter->data);
+      midi_note_reset_counterpart (
+        mnw->midi_note, reset_trans);
+    }
+  g_list_free (children);
+}
+
 /**
  * Sets transient notes and actual notes
  * visibility based on the current action.
+ *
+ * Loops through all currently shown notes.
  */
 void
 midi_arranger_widget_update_visibility (
   MidiArrangerWidget * self)
 {
-  ARRANGER_SET_OBJ_VISIBILITY_ARRAY (
-    MA_SELECTIONS->midi_notes,
-    MA_SELECTIONS->num_midi_notes,
-    MidiNote,
-    midi_note);
+  GList *children, *iter;
+  children =
+    gtk_container_get_children (
+      GTK_CONTAINER (self));
+  MidiNoteWidget * mnw = NULL;
+  for (iter = children;
+       iter != NULL;
+       iter = g_list_next (iter))
+    {
+      if (!Z_IS_MIDI_NOTE_WIDGET (iter->data))
+        continue;
+
+      mnw =
+        Z_MIDI_NOTE_WIDGET (iter->data);
+      arranger_object_info_set_widget_visibility_and_state (
+        &mnw->midi_note->obj_info, 1);
+    }
+  g_list_free (children);
 }
 
 void
@@ -862,9 +909,7 @@ midi_arranger_widget_move_items_x (
 {
   midi_arranger_selections_add_ticks (
     MA_SELECTIONS, ticks_diff, F_USE_CACHED,
-    copy_moving ?
-      AO_UPDATE_TRANS :
-      AO_UPDATE_ALL);
+    AO_UPDATE_NON_TRANS);
 }
 
 
@@ -912,35 +957,36 @@ midi_arranger_widget_move_items_y (
   ARRANGER_WIDGET_GET_PRIVATE(self);
 
   int y_delta;
-  int ar_start_val =
-    midi_note_get_main_trans_midi_note (
-      MA_SELECTIONS->midi_notes[0])->val;
-  int ar_end_val =
+  /* first note selected */
+  int first_note_selected =
+    midi_note_get_main_midi_note (
+      self->start_midi_note)->val;
+  /* note at cursor */
+  int note_at_cursor =
     midi_arranger_widget_get_note_at_y (
       ar_prv->start_y + offset_y);
 
-  y_delta = ar_end_val - ar_start_val;
+  y_delta = note_at_cursor - first_note_selected;
   y_delta =
     calc_deltamax_for_note_movement (y_delta);
-  if (ar_end_val != ar_start_val)
-    {
-      MidiNote * midi_note;
-      for (int i = 0;
-           i < MA_SELECTIONS->
-                 num_midi_notes;
-           i++)
-        {
-          midi_note =
-            midi_note_get_main_trans_midi_note (
-              MA_SELECTIONS->midi_notes[i]);
-          midi_note_set_val (
-            midi_note, midi_note->val + y_delta,
-            AO_UPDATE_ALL);
-          if (midi_note->widget)
-            midi_note_widget_update_tooltip (
-              midi_note->widget, 0);
+  MidiNote * midi_note;
+  ArrangerObjectUpdateFlag flag =
+    AO_UPDATE_NON_TRANS;
 
-        }
+  for (int i = 0;
+       i < MA_SELECTIONS->
+             num_midi_notes;
+       i++)
+    {
+      midi_note =
+        midi_note_get_main_midi_note (
+          MA_SELECTIONS->midi_notes[i]);
+      midi_note_set_val (
+        midi_note, midi_note->val + y_delta,
+        flag);
+      if (midi_note->widget)
+        midi_note_widget_update_tooltip (
+          midi_note->widget, 0);
     }
 }
 
@@ -987,6 +1033,8 @@ midi_arranger_widget_on_drag_end (
           trans_note->cache_start_pos.total_ticks);
       undo_manager_perform (
         UNDO_MANAGER, ua);
+      midi_arranger_selections_reset_counterparts (
+        MA_SELECTIONS, 1);
     }
   else if (ar_prv->action ==
         UI_OVERLAY_ACTION_RESIZING_R)
@@ -994,9 +1042,6 @@ midi_arranger_widget_on_drag_end (
       MidiNote * trans_note =
         midi_note_get_main_trans_midi_note (
           MA_SELECTIONS->midi_notes[0]);
-      /*g_message ("posses");*/
-      /*position_print_simple (&trans_note->end_pos);*/
-      /*position_print_simple (&trans_note->cache_end_pos);*/
       UndoableAction * ua =
         (UndoableAction *)
         emas_action_new_resize_r (
@@ -1005,6 +1050,8 @@ midi_arranger_widget_on_drag_end (
           trans_note->cache_end_pos.total_ticks);
       undo_manager_perform (
         UNDO_MANAGER, ua);
+      midi_arranger_selections_reset_counterparts (
+        MA_SELECTIONS, 1);
     }
   else if (ar_prv->action ==
         UI_OVERLAY_ACTION_STARTING_MOVING)
@@ -1025,18 +1072,23 @@ midi_arranger_widget_on_drag_end (
   else if (ar_prv->action ==
              UI_OVERLAY_ACTION_MOVING)
     {
-      MidiNote * trans_note =
-        midi_note_get_main_trans_midi_note (
+      MidiNote * note =
+        midi_note_get_main_midi_note (
           MA_SELECTIONS->midi_notes[0]);
+      long ticks_diff =
+        note->start_pos.total_ticks -
+          note->cache_start_pos.total_ticks;
+      int pitch_diff =
+        note->val - note->cache_val;
       UndoableAction * ua =
         move_midi_arranger_selections_action_new (
           MA_SELECTIONS,
-          trans_note->start_pos.total_ticks -
-            trans_note->cache_start_pos.total_ticks,
-          trans_note->val -
-            trans_note->cache_val);
+          ticks_diff,
+          pitch_diff);
       undo_manager_perform (
         UNDO_MANAGER, ua);
+      midi_arranger_selections_reset_counterparts (
+        MA_SELECTIONS, 1);
     }
   /* if copy/link-moved */
   else if (ar_prv->action ==
@@ -1044,21 +1096,25 @@ midi_arranger_widget_on_drag_end (
            ar_prv->action ==
              UI_OVERLAY_ACTION_MOVING_LINK)
     {
-      MidiNote * trans_note =
-        midi_note_get_main_trans_midi_note (
+      MidiNote * note =
+        midi_note_get_main_midi_note (
           MA_SELECTIONS->midi_notes[0]);
+      long ticks_diff =
+        note->start_pos.total_ticks -
+          note->cache_start_pos.total_ticks;
+      int pitch_diff =
+        note->val - note->cache_val;
+      midi_arranger_selections_reset_counterparts (
+        MA_SELECTIONS, 0);
       UndoableAction * ua =
         (UndoableAction *)
         duplicate_midi_arranger_selections_action_new (
           MA_SELECTIONS,
-          trans_note->start_pos.total_ticks -
-            trans_note->cache_start_pos.total_ticks,
-          trans_note->val -
-            trans_note->cache_val);
-      midi_arranger_selections_clear (
-        MA_SELECTIONS);
+          ticks_diff,
+          pitch_diff);
       undo_manager_perform (
         UNDO_MANAGER, ua);
+      /* TODO modifier too */
     }
   else if (ar_prv->action ==
              UI_OVERLAY_ACTION_NONE)
@@ -1087,6 +1143,8 @@ midi_arranger_widget_on_drag_end (
   midi_modifier_arranger_widget_update_visibility (
     MW_MIDI_MODIFIER_ARRANGER);
 
+  midi_arranger_widget_refresh_children (self);
+
   self->start_midi_note = NULL;
   /*if (self->start_midi_note_clone)*/
     /*{*/
@@ -1193,144 +1251,6 @@ on_focus (GtkWidget       *widget,
   return FALSE;
 }
 
-void
-midi_arranger_widget_auto_scroll (
-  MidiArrangerWidget * self,
-  GtkScrolledWindow *  scrolled_window,
-  int                  transient)
-{
-  return;
-  Region * region = CLIP_EDITOR->region;
-  int scroll_speed = 20;
-  int border_distance = 10;
-  g_message ("midi auto scrolling");
-  if (region != 0)
-  {
-    MidiNote * first_note =
-      midi_arranger_selections_get_first_midi_note (
-        MA_SELECTIONS, transient);
-    MidiNote * last_note =
-      midi_arranger_selections_get_last_midi_note (
-        MA_SELECTIONS, transient);
-    MidiNote * lowest_note =
-      midi_arranger_selections_get_lowest_note (
-        MA_SELECTIONS, transient);
-    MidiNote * highest_note =
-      midi_arranger_selections_get_highest_note (
-        MA_SELECTIONS, transient);
-    g_warn_if_fail (first_note && last_note &&
-              lowest_note && highest_note &&
-              first_note->widget &&
-              last_note->widget &&
-              lowest_note->widget &&
-              highest_note->widget);
-    int arranger_width =
-      gtk_widget_get_allocated_width (
-        GTK_WIDGET (scrolled_window));
-    int arranger_height =
-      gtk_widget_get_allocated_height (
-        GTK_WIDGET (scrolled_window));
-    GtkAdjustment *hadj =
-      gtk_scrolled_window_get_hadjustment (
-        GTK_SCROLLED_WINDOW (scrolled_window));
-    GtkAdjustment *vadj =
-      gtk_scrolled_window_get_vadjustment (
-        GTK_SCROLLED_WINDOW (scrolled_window));
-    int v_delta = 0;
-    int h_delta = 0;
-    if (lowest_note != 0)
-    {
-      GtkWidget *focused =
-        GTK_WIDGET (lowest_note->widget);
-      gint note_x, note_y;
-      gtk_widget_translate_coordinates (
-        focused,
-        GTK_WIDGET (scrolled_window),
-        0,
-        0,
-        &note_x,
-        &note_y);
-      int note_height =
-        gtk_widget_get_allocated_height (
-          GTK_WIDGET (focused));
-      if (note_y + note_height + border_distance
-        >= arranger_height)
-      {
-        v_delta = scroll_speed;
-      }
-    }
-    if (highest_note != 0)
-    {
-      GtkWidget *focused = GTK_WIDGET (
-        highest_note->widget);
-      gint note_x, note_y;
-      gtk_widget_translate_coordinates (
-        focused,
-        GTK_WIDGET (scrolled_window),
-        0,
-        0,
-        &note_x,
-        &note_y);
-      /*int note_height = gtk_widget_get_allocated_height (*/
-        /*GTK_WIDGET (focused));*/
-      if (note_y - border_distance <= 1)
-      {
-        v_delta = scroll_speed * -1;
-      }
-    }
-    if (first_note != 0)
-    {
-      gint note_x, note_y;
-      GtkWidget *focused =
-        GTK_WIDGET (first_note->widget);
-      gtk_widget_translate_coordinates (
-        focused,
-        GTK_WIDGET (scrolled_window),
-        0,
-        0,
-        &note_x,
-        &note_y);
-      /*int note_width = gtk_widget_get_allocated_width (*/
-        /*GTK_WIDGET (focused));*/
-      if (note_x - border_distance <= 10)
-      {
-        h_delta = scroll_speed * -1;
-      }
-    }
-    if (last_note != 0)
-    {
-      gint note_x, note_y;
-      GtkWidget *focused = GTK_WIDGET (last_note->widget);
-      gtk_widget_translate_coordinates (
-        focused,
-        GTK_WIDGET (scrolled_window),
-        0,
-        0,
-        &note_x,
-        &note_y);
-      int note_width = gtk_widget_get_allocated_width (
-        GTK_WIDGET (focused));
-      if (note_x + note_width + border_distance
-        > arranger_width)
-      {
-        h_delta = scroll_speed;
-      }
-    }
-    if (h_delta != 0)
-    {
-      gtk_adjustment_set_value (
-        hadj,
-        gtk_adjustment_get_value (hadj) + h_delta);
-    }
-    if (v_delta != 0)
-    {
-      gtk_adjustment_set_value (
-        vadj,
-        gtk_adjustment_get_value (vadj) + v_delta);
-    }
-  }
-}
-
 static void
 midi_arranger_widget_class_init (
   MidiArrangerWidgetClass * klass)
