@@ -22,7 +22,7 @@
  *
  */
 
-#include "audio/bus_track.h"
+#include "audio/audio_bus_track.h"
 #include "audio/channel.h"
 #include "audio/chord_track.h"
 #include "audio/instrument_track.h"
@@ -139,7 +139,8 @@ midi_note_draw_cb (
           cairo_set_source_rgba (
             cr, color.red + 0.4,
             color.green + 0.2,
-            color.blue + 0.2, 1);
+            color.blue + 0.2,
+            DEBUGGING ? 0.3 : 1);
         }
       if (PIANO_ROLL->drum_mode)
         {
@@ -174,8 +175,8 @@ midi_note_draw_cb (
     g_strdup_printf (
       "%s<sup>%d</sup>",
       chord_descriptor_note_to_string (
-        self->midi_note->val % 12),
-      self->midi_note->val / 12 - 2);
+        mn->val % 12),
+      mn->val / 12 - 1);
   if (DEBUGGING &&
       midi_note_is_transient (self->midi_note))
     {
@@ -191,7 +192,7 @@ midi_note_draw_cb (
     &color, &c2);
   gdk_cairo_set_source_rgba (cr, &c2);
   if (DEBUGGING || !PIANO_ROLL->drum_mode)
-    z_cairo_draw_text (cr, str);
+    z_cairo_draw_text (cr, widget, str);
   g_free (str);
 
  return FALSE;
@@ -284,7 +285,7 @@ midi_note_widget_update_tooltip (
     g_strdup_printf (
       "[%s%d] %d",
       note,
-      self->midi_note->val / 12 - 2,
+      self->midi_note->val / 12 - 1,
       self->midi_note->vel->vel);
   gtk_widget_set_tooltip_text (
     GTK_WIDGET (self), tooltip);
@@ -317,10 +318,6 @@ on_destroy (
   g_message ("on destroy %p", widget);
 }
 
-DEFINE_ARRANGER_OBJECT_WIDGET_SELECT (
-  MidiNote, midi_note,
-  midi_arranger_selections, MA_SELECTIONS);
-
 /**
  * Destroys the widget completely.
  */
@@ -355,7 +352,7 @@ midi_note_widget_new (MidiNote * midi_note)
       "[%s%d] %d",
       chord_descriptor_note_to_string (
         self->midi_note->val % 12),
-      self->midi_note->val / 12 - 2,
+      self->midi_note->val / 12 - 1,
       self->midi_note->vel->vel);
   gtk_widget_set_tooltip_text (
     GTK_WIDGET (self), tooltip);
