@@ -34,7 +34,7 @@
 #include "audio/automation_track.h"
 #include "audio/automation_tracklist.h"
 #include "audio/audio_track.h"
-#include "audio/bus_track.h"
+#include "audio/audio_bus_track.h"
 #include "audio/channel.h"
 #include "audio/chord_object.h"
 #include "audio/chord_region.h"
@@ -57,6 +57,8 @@
 #include "gui/widgets/center_dock.h"
 #include "gui/widgets/chord_object.h"
 #include "gui/widgets/color_area.h"
+#include "gui/widgets/export_midi_file_dialog.h"
+#include "gui/widgets/foldable_notebook.h"
 #include "gui/widgets/inspector.h"
 #include "gui/widgets/main_window.h"
 #include "gui/widgets/marker.h"
@@ -419,7 +421,7 @@ timeline_arranger_widget_get_track_lane_at_y (
             continue;
 
           if (ui_is_child_hit (
-                GTK_CONTAINER (self),
+                GTK_WIDGET (self),
                 GTK_WIDGET (lane->widget),
                 0, 1, 0, y, 0, 0))
             return lane;
@@ -439,8 +441,11 @@ timeline_arranger_widget_get_track_at_y (
     {
       track = TRACKLIST->tracks[i];
 
+      if (!track->visible)
+        continue;
+
       if (ui_is_child_hit (
-            GTK_CONTAINER (self),
+            GTK_WIDGET (self),
             GTK_WIDGET (track->widget),
             0, 1, 0, y, 0, 0))
         return track;
@@ -482,7 +487,7 @@ timeline_arranger_widget_get_automation_track_at_y (
           /* TODO check the rest */
           if (at->visible && at->widget &&
               ui_is_child_hit (
-                GTK_CONTAINER (MW_TIMELINE),
+                GTK_WIDGET (MW_TIMELINE),
                 GTK_WIDGET (at->widget),
                 0, 1, 0, y, 0, 0))
             return at;
@@ -555,8 +560,8 @@ timeline_arranger_widget_select_all (
                 {
                   r = lane->regions[k];
 
-                  region_widget_select (
-                    r->widget, select);
+                  region_select (
+                    r, select);
                 }
             }
         }
@@ -576,8 +581,8 @@ timeline_arranger_widget_select_all (
             {
               r = at->regions[k];
 
-              region_widget_select (
-                r->widget, select);
+              region_select (
+                r, select);
             }
         }
     }
@@ -588,8 +593,8 @@ timeline_arranger_widget_select_all (
        i < P_CHORD_TRACK->num_scales; i++)
     {
       scale = P_CHORD_TRACK->scales[i];
-      scale_object_widget_select (
-        scale->widget, select);
+      scale_object_select (
+        scale, select);
     }
 
   /**
@@ -601,6 +606,37 @@ timeline_arranger_widget_select_all (
     }
 }
 
+static void
+on_export_as_midi_file_clicked (
+  GtkMenuItem * menuitem,
+  Region *      r)
+{
+  GtkDialog * dialog =
+    GTK_DIALOG (
+      export_midi_file_dialog_widget_new_for_region (
+        GTK_WINDOW (MAIN_WINDOW),
+        r));
+  int res = gtk_dialog_run (dialog);
+  char * filename;
+  switch (res)
+    {
+    case GTK_RESPONSE_ACCEPT:
+      // do_application_specific_something ();
+      filename =
+        gtk_file_chooser_get_filename (
+          GTK_FILE_CHOOSER (dialog));
+      g_message ("exporting to %s", filename);
+      midi_region_export_to_midi_file (
+        r, filename, 0);
+      g_free (filename);
+      break;
+    default:
+      // do_nothing_since_dialog_was_cancelled ();
+      break;
+    }
+  gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
 /**
  * Shows context menu.
  *
@@ -614,24 +650,34 @@ timeline_arranger_widget_show_context_menu (
 {
   GtkWidget *menu, *menuitem;
 
-  /*RegionWidget * clicked_region =*/
-    /*timeline_arranger_widget_get_hit_region (*/
-      /*self, x, y);*/
-  /*AutomationPointWidget * clicked_ap =*/
-    /*timeline_arranger_widget_get_hit_ap (*/
-      /*self, x, y);*/
-  /*AutomationCurveWidget * ac =*/
-    /*timeline_arranger_widget_get_hit_curve (*/
-      /*self, x, y);*/
+  RegionWidget * clicked_region =
+    timeline_arranger_widget_get_hit_region (
+      self, x, y);
+  Region * r = NULL;
+  if (clicked_region)
+    {
+      REGION_WIDGET_GET_PRIVATE (clicked_region);
+      r = rw_prv->region;
+    }
 
   menu = gtk_menu_new();
 
-  menuitem = gtk_menu_item_new_with_label("Do something");
-
-  /*g_signal_connect(menuitem, "activate",*/
-                   /*(GCallback) view_popup_menu_onDoSomething, treeview);*/
-
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+  if (r)
+    {
+      if (r->type == REGION_TYPE_MIDI)
+        {
+          menuitem =
+            gtk_menu_item_new_with_label (
+              "Export as MIDI file");
+          gtk_menu_shell_append (
+            GTK_MENU_SHELL(menu), menuitem);
+          g_signal_connect (
+            menuitem, "activate",
+            G_CALLBACK (
+              on_export_as_midi_file_clicked),
+            r);
+        }
+    }
 
   gtk_widget_show_all(menu);
 
@@ -681,7 +727,16 @@ timeline_arranger_widget_on_drag_begin_region_hit (
   /* if double click bring up piano roll */
   if (ar_prv->n_press == 2 &&
       !ar_prv->ctrl_held)
-    SHOW_CLIP_EDITOR;
+    {
+      /* set the bot panel visible */
+      gtk_widget_set_visible (
+        GTK_WIDGET (
+          MW_BOT_DOCK_EDGE), 1);
+      foldable_notebook_widget_set_visibility (
+        MW_BOT_FOLDABLE_NOTEBOOK, 1);
+
+      SHOW_CLIP_EDITOR;
+    }
 
   /* get x as local to the region */
   gint wx, wy;
@@ -2108,7 +2163,7 @@ add_children_from_marker_track (
 }
 
 static inline void
-add_children_from_instrument_track (
+add_children_from_midi_track (
   TimelineArrangerWidget * self,
   Track *                  it)
 {
@@ -2182,9 +2237,9 @@ add_children_from_master_track (
 }
 
 static void
-add_children_from_bus_track (
+add_children_from_audio_bus_track (
   TimelineArrangerWidget * self,
-  BusTrack *               mt)
+  AudioBusTrack *               mt)
 {
   ChannelTrack * ct = (ChannelTrack *) mt;
   add_children_from_channel_track (ct);
@@ -2268,9 +2323,12 @@ timeline_arranger_widget_refresh_children (
                 self, track);
               break;
             case TRACK_TYPE_INSTRUMENT:
-              add_children_from_instrument_track (
-                self,
-                (InstrumentTrack *) track);
+              add_children_from_midi_track (
+                self, track);
+              break;
+            case TRACK_TYPE_MIDI:
+              add_children_from_midi_track (
+                self, track);
               break;
             case TRACK_TYPE_MASTER:
               add_children_from_master_track (
@@ -2282,16 +2340,16 @@ timeline_arranger_widget_refresh_children (
                 self,
                 (AudioTrack *) track);
               break;
-            case TRACK_TYPE_BUS:
-              add_children_from_bus_track (
+            case TRACK_TYPE_AUDIO_BUS:
+              add_children_from_audio_bus_track (
                 self,
-                (BusTrack *) track);
+                (AudioBusTrack *) track);
               break;
             case TRACK_TYPE_MARKER:
               add_children_from_marker_track (
                 self, track);
               break;
-            case TRACK_TYPE_GROUP:
+            case TRACK_TYPE_AUDIO_GROUP:
               /* TODO */
               break;
             default:
