@@ -17,10 +17,13 @@
  * along with Zrythm.  If not, see <https://www.gnu.org/licenses/>.
  */
 
+#include "actions/copy_tracks_action.h"
+#include "actions/move_tracks_action.h"
 #include "audio/automatable.h"
 #include "audio/automation_track.h"
-#include "audio/bus_track.h"
-#include "audio/group_track.h"
+#include "audio/audio_bus_track.h"
+#include "audio/audio_group_track.h"
+#include "audio/master_track.h"
 #include "audio/instrument_track.h"
 #include "audio/track.h"
 #include "audio/tracklist.h"
@@ -31,17 +34,20 @@
 #include "gui/widgets/automation_track.h"
 #include "gui/widgets/automation_tracklist.h"
 #include "gui/widgets/bot_bar.h"
-#include "gui/widgets/bus_track.h"
+#include "gui/widgets/bot_dock_edge.h"
+#include "gui/widgets/audio_bus_track.h"
 #include "gui/widgets/center_dock.h"
 #include "gui/widgets/color_area.h"
 #include "gui/widgets/chord_track.h"
-#include "gui/widgets/group_track.h"
+#include "gui/widgets/editable_label.h"
+#include "gui/widgets/audio_group_track.h"
 #include "gui/widgets/instrument_track.h"
 #include "gui/widgets/main_window.h"
 #include "gui/widgets/marker_track.h"
 #include "gui/widgets/master_track.h"
 #include "gui/widgets/midi_activity_bar.h"
 #include "gui/widgets/midi_track.h"
+#include "gui/widgets/mixer.h"
 #include "gui/widgets/timeline_arranger.h"
 #include "gui/widgets/timeline_bg.h"
 #include "gui/widgets/track.h"
@@ -94,9 +100,7 @@ track_widget_select (
     {
     case TRACK_TYPE_INSTRUMENT:
     case TRACK_TYPE_AUDIO:
-    case TRACK_TYPE_MASTER:
-    case TRACK_TYPE_BUS:
-    case TRACK_TYPE_GROUP:
+    case TRACK_TYPE_MIDI:
       chan = track->channel;
       g_message (
         "%sselecting track %s, recording %d sa %d",
@@ -121,6 +125,11 @@ track_widget_select (
       track_widget_refresh (self);
       break;
     case TRACK_TYPE_CHORD:
+    case TRACK_TYPE_MASTER:
+    case TRACK_TYPE_AUDIO_BUS:
+    case TRACK_TYPE_AUDIO_GROUP:
+    case TRACK_TYPE_MIDI_BUS:
+    case TRACK_TYPE_MIDI_GROUP:
       break;
     default:
       break;
@@ -188,8 +197,9 @@ track_widget_refresh (TrackWidget * self)
       REFRESH_TW (MASTER, master);
       REFRESH_TW (AUDIO, audio);
       REFRESH_TW (CHORD, chord);
-      REFRESH_TW (BUS, bus);
-      REFRESH_TW (GROUP, group);
+      REFRESH_TW (MIDI, midi);
+      REFRESH_TW (AUDIO_BUS, audio_bus);
+      REFRESH_TW (AUDIO_GROUP, audio_group);
       REFRESH_TW (MARKER, marker);
     default:
       break;
@@ -218,8 +228,8 @@ track_widget_refresh_buttons (
       REFRESH_TW_BUTTONS (MASTER, master);
       REFRESH_TW_BUTTONS (AUDIO, audio);
       REFRESH_TW_BUTTONS (CHORD, chord);
-      REFRESH_TW_BUTTONS (BUS, bus);
-      REFRESH_TW_BUTTONS (GROUP, group);
+      REFRESH_TW_BUTTONS (AUDIO_BUS, audio_bus);
+      REFRESH_TW_BUTTONS (AUDIO_GROUP, audio_group);
       REFRESH_TW_BUTTONS (MARKER, marker);
     default:
       break;
@@ -322,10 +332,10 @@ track_widget_block_all_signal_handlers (
         mtw->mute,
         tw_prv->mute_toggled_handler_id);
     }
-  else if (Z_IS_BUS_TRACK_WIDGET (self))
+  else if (Z_IS_AUDIO_BUS_TRACK_WIDGET (self))
     {
-      BusTrackWidget * mtw =
-        Z_BUS_TRACK_WIDGET (self);
+      AudioBusTrackWidget * mtw =
+        Z_AUDIO_BUS_TRACK_WIDGET (self);
       g_signal_handler_block (
         mtw->solo,
         tw_prv->solo_toggled_handler_id);
@@ -387,10 +397,10 @@ track_widget_unblock_all_signal_handlers (
         mtw->mute,
         tw_prv->mute_toggled_handler_id);
     }
-  else if (Z_IS_BUS_TRACK_WIDGET (self))
+  else if (Z_IS_AUDIO_BUS_TRACK_WIDGET (self))
     {
-      BusTrackWidget * mtw =
-        Z_BUS_TRACK_WIDGET (self);
+      AudioBusTrackWidget * mtw =
+        Z_AUDIO_BUS_TRACK_WIDGET (self);
       g_signal_handler_unblock (
         mtw->solo,
         tw_prv->solo_toggled_handler_id);
@@ -434,11 +444,10 @@ show_context_menu (TrackWidget * self)
           /* delete track */
           if (num_selected == 1)
             str =
-              g_strdup_printf (_("_Delete Track"));
+              g_strdup (_("_Delete Track"));
           else
             str =
-              g_strdup_printf (_("_Delete %d Tracks"),
-                               num_selected);
+              g_strdup (_("_Delete Tracks"));
           menuitem =
             z_gtk_create_menu_item (
               str,
@@ -453,12 +462,10 @@ show_context_menu (TrackWidget * self)
           /* duplicate track */
           if (num_selected == 1)
             str =
-              g_strdup_printf (_("_Duplicate Track"));
+              g_strdup (_("_Duplicate Track"));
           else
             str =
-              g_strdup_printf (
-                _("_Duplicate %d Tracks"),
-                num_selected);
+              g_strdup (_("_Duplicate Tracks"));
           menuitem =
             z_gtk_create_menu_item (
               str,
@@ -484,6 +491,18 @@ show_context_menu (TrackWidget * self)
               "win.duplicate-selected-tracks");
           APPEND (menuitem);
         }
+
+      menuitem =
+        z_gtk_create_menu_item (
+          num_selected == 1 ?
+            _("Hide Track") :
+            _("Hide Tracks"),
+          "z-gnumeric-column-hide",
+          0,
+          NULL,
+          0,
+          "win.hide-selected-tracks");
+      APPEND (menuitem);
     }
 
 #undef APPEND
@@ -563,6 +582,275 @@ multipress_pressed (GtkGestureMultiPress *gesture,
       F_APPEND : F_NO_APPEND);
 }
 
+static void
+on_drag_begin (GtkGestureDrag *gesture,
+               gdouble         start_x,
+               gdouble         start_y,
+               TrackWidget * self)
+{
+  g_message ("drag begin ch");
+  TRACK_WIDGET_GET_PRIVATE (self);
+
+  tw_prv->selected_in_dnd = 0;
+  tw_prv->dragged = 0;
+}
+
+static void
+on_drag_update (GtkGestureDrag * gesture,
+               gdouble         offset_x,
+               gdouble         offset_y,
+               TrackWidget * self)
+{
+  g_message ("drag_update");
+  TRACK_WIDGET_GET_PRIVATE (self);
+
+  tw_prv->dragged = 1;
+}
+
+static void
+on_drag_data_received (
+  GtkWidget        *widget,
+  GdkDragContext   *context,
+  gint              x,
+  gint              y,
+  GtkSelectionData *data,
+  guint             info,
+  guint             time,
+  TrackWidget * self)
+{
+  g_message ("drag data received");
+  TRACK_WIDGET_GET_PRIVATE (self);
+  Track * this = tw_prv->track;
+
+  /* determine if moving or copying */
+  GdkDragAction action =
+    gdk_drag_context_get_selected_action (
+      context);
+
+  tracklist_selections_gprint (
+    TRACKLIST_SELECTIONS);
+
+  int h =
+    gtk_widget_get_allocated_height (widget);
+
+  /* determine position to move to */
+  int pos;
+  if (y < h / 2)
+    {
+      if (this->pos <=
+          MW_MIXER->start_drag_track->pos)
+        pos = this->pos;
+      else
+        {
+          Track * prev =
+            tracklist_get_prev_visible_track (
+              TRACKLIST, this);
+          pos =
+            prev ? prev->pos : this->pos;
+        }
+    }
+  else
+    {
+      if (this->pos >=
+          MW_MIXER->start_drag_track->pos)
+        pos = this->pos;
+      else
+        {
+          Track * next =
+            tracklist_get_next_visible_track (
+              TRACKLIST, this);
+          pos =
+            next ? next->pos : this->pos;
+        }
+    }
+
+  UndoableAction * ua = NULL;
+  if (action == GDK_ACTION_COPY)
+    {
+      ua =
+        copy_tracks_action_new (
+          TRACKLIST_SELECTIONS, pos);
+    }
+  else if (action == GDK_ACTION_MOVE)
+    {
+      ua =
+        move_tracks_action_new (
+          TRACKLIST_SELECTIONS, pos);
+    }
+
+  g_warn_if_fail (ua);
+
+  undo_manager_perform (
+    UNDO_MANAGER, ua);
+}
+
+static void
+on_drag_data_get (
+  GtkWidget        *widget,
+  GdkDragContext   *context,
+  GtkSelectionData *data,
+  guint             info,
+  guint             time,
+  TrackWidget * self)
+{
+  g_message ("drag data get");
+
+  /* Not really needed since the selections are
+   * used. just send master */
+  gtk_selection_data_set (
+    data,
+    gdk_atom_intern_static_string (
+      TARGET_ENTRY_TRACK),
+    32,
+    (const guchar *) P_MASTER_TRACK,
+    sizeof (Track));
+}
+
+/**
+ * For drag n drop.
+ */
+static void
+on_dnd_drag_begin (
+  GtkWidget      *widget,
+  GdkDragContext *context,
+  TrackWidget * self)
+{
+  g_message ("dnd drag begin");
+  TRACK_WIDGET_GET_PRIVATE (self);
+
+  Track * track = tw_prv->track;
+  tw_prv->selected_in_dnd = 1;
+  MW_MIXER->start_drag_track = track;
+
+  if (tw_prv->n_press == 1)
+    {
+      int ctrl = 0, selected = 0;
+
+      ctrl = tw_prv->ctrl_held_at_start;
+
+      if (tracklist_selections_contains_track (
+            TRACKLIST_SELECTIONS,
+            track))
+        selected = 1;
+
+      /* no control & not selected */
+      if (!ctrl && !selected)
+        {
+          tracklist_selections_select_single (
+            TRACKLIST_SELECTIONS,
+            track);
+        }
+      else if (!ctrl && selected)
+        { }
+      else if (ctrl && !selected)
+        tracklist_selections_add_track (
+          TRACKLIST_SELECTIONS,
+          track);
+    }
+}
+
+/**
+ * Highlights/unhighlights the Tracks
+ * appropriately.
+ */
+static void
+do_highlight (
+  TrackWidget * self,
+  gint x,
+  gint y)
+{
+  TRACK_WIDGET_GET_PRIVATE (self);
+
+  /* if we are closer to the start of selection than
+   * the end */
+  int h =
+    gtk_widget_get_allocated_height (
+      GTK_WIDGET (self));
+  if (y < h / 2)
+    {
+      /* highlight top */
+      gtk_drag_highlight (
+        GTK_WIDGET (
+          tw_prv->highlight_top_box));
+      gtk_widget_set_size_request (
+        GTK_WIDGET (
+          tw_prv->highlight_top_box),
+        -1, 2);
+
+      /* unhilight bot */
+      gtk_drag_unhighlight (
+        GTK_WIDGET (
+          tw_prv->highlight_bot_box));
+      gtk_widget_set_size_request (
+        GTK_WIDGET (
+          tw_prv->highlight_bot_box),
+        -1, -1);
+    }
+  else
+    {
+      /* highlight bot */
+      gtk_drag_highlight (
+        GTK_WIDGET (
+          tw_prv->highlight_bot_box));
+      gtk_widget_set_size_request (
+        GTK_WIDGET (
+          tw_prv->highlight_bot_box),
+        -1, 2);
+
+      /* unhilight top */
+      gtk_drag_unhighlight (
+        GTK_WIDGET (
+          tw_prv->highlight_top_box));
+      gtk_widget_set_size_request (
+        GTK_WIDGET (
+          tw_prv->highlight_top_box),
+        -1, -1);
+    }
+}
+
+static void
+on_drag_motion (
+  GtkWidget *widget,
+  GdkDragContext *context,
+  gint x,
+  gint y,
+  guint time,
+  TrackWidget * self)
+{
+  GdkModifierType mask;
+  z_gtk_widget_get_mask (
+    widget, &mask);
+  if (mask & GDK_CONTROL_MASK)
+    gdk_drag_status (context, GDK_ACTION_COPY, time);
+  else
+    gdk_drag_status (context, GDK_ACTION_MOVE, time);
+
+  do_highlight (self, x, y);
+}
+
+static void
+on_drag_leave (
+  GtkWidget      *widget,
+  GdkDragContext *context,
+  guint           time,
+  TrackWidget * self)
+{
+  g_message ("on_drag_leave");
+  TRACK_WIDGET_GET_PRIVATE (self);
+
+  /*do_highlight (self);*/
+  gtk_drag_unhighlight (
+    GTK_WIDGET (tw_prv->highlight_top_box));
+  gtk_widget_set_size_request (
+    GTK_WIDGET (tw_prv->highlight_top_box),
+    -1, -1);
+  gtk_drag_unhighlight (
+    GTK_WIDGET (tw_prv->highlight_bot_box));
+  gtk_widget_set_size_request (
+    GTK_WIDGET (tw_prv->highlight_bot_box),
+    -1, -1);
+}
+
 /**
  * Wrapper for child track widget.
  *
@@ -584,8 +872,8 @@ track_widget_new (Track * track)
   switch (track->type)
     {
     NEW_TW (CHORD, chord);
-    NEW_TW (BUS, bus);
-    NEW_TW (GROUP, group);
+    NEW_TW (AUDIO_BUS, audio_bus);
+    NEW_TW (AUDIO_GROUP, audio_group);
     NEW_TW (INSTRUMENT, instrument);
     NEW_TW (MASTER, master);
     NEW_TW (AUDIO, audio);
@@ -605,6 +893,9 @@ track_widget_new (Track * track)
   midi_activity_bar_widget_setup_track (
     tw_prv->top_grid->midi_activity,
     track);
+  editable_label_widget_setup (
+    tw_prv->top_grid->name, track,
+    track_get_name, track_set_name);
 
   tw_prv->track = track;
 
@@ -708,6 +999,20 @@ track_widget_get_bottom_paned (TrackWidget * self)
     DZL_MULTI_PANED (tw_prv->paned), 1);
 }
 
+/**
+ * Sets the Track name on the TrackWidget.
+ */
+void
+track_widget_set_name (
+  TrackWidget * self,
+  const char * name)
+{
+  TRACK_WIDGET_GET_PRIVATE (self);
+
+  gtk_label_set_text (
+    tw_prv->top_grid->name->label, name);
+}
+
 static void
 on_destroy (
   TrackWidget * self)
@@ -731,6 +1036,22 @@ track_widget_init (TrackWidget * self)
 
   TRACK_WIDGET_GET_PRIVATE (self);
 
+  /* set font sizes */
+  gtk_label_set_max_width_chars (
+    tw_prv->top_grid->name->label, 14);
+  gtk_label_set_width_chars (
+    tw_prv->top_grid->name->label, 14);
+  gtk_label_set_ellipsize (
+    tw_prv->top_grid->name->label, PANGO_ELLIPSIZE_END);
+  gtk_label_set_xalign (
+    tw_prv->top_grid->name->label, 0);
+
+  tw_prv->drag =
+    GTK_GESTURE_DRAG (
+      gtk_gesture_drag_new (
+        GTK_WIDGET (
+          self)));
+
   tw_prv->multipress =
     GTK_GESTURE_MULTI_PRESS (
       gtk_gesture_multi_press_new (GTK_WIDGET (self)));
@@ -745,6 +1066,30 @@ track_widget_init (TrackWidget * self)
   gtk_widget_add_events (GTK_WIDGET (self),
                          GDK_ALL_EVENTS_MASK);
 
+  GtkTargetEntry entries[1];
+  entries[0].target = TARGET_ENTRY_TRACK;
+  entries[0].flags = GTK_TARGET_SAME_APP;
+  entries[0].info =
+    symap_map (ZSYMAP, TARGET_ENTRY_TRACK);
+
+  /* set as drag source for track */
+  gtk_drag_source_set (
+    GTK_WIDGET (tw_prv->event_box),
+    GDK_BUTTON1_MASK,
+    entries,
+    1,
+    GDK_ACTION_MOVE | GDK_ACTION_COPY);
+  /* set as drag dest for track (the track will
+   * be moved based on which half it was dropped in,
+   * top or bot) */
+  gtk_drag_dest_set (
+    GTK_WIDGET (self),
+    GTK_DEST_DEFAULT_MOTION |
+      GTK_DEST_DEFAULT_DROP,
+    entries,
+    1,
+    GDK_ACTION_MOVE | GDK_ACTION_COPY);
+
   g_signal_connect (
     G_OBJECT (tw_prv->multipress), "pressed",
     G_CALLBACK (multipress_pressed), self);
@@ -763,6 +1108,29 @@ track_widget_init (TrackWidget * self)
     G_OBJECT(tw_prv->event_box),
     "motion-notify-event",
     G_CALLBACK (on_motion),  self);
+  g_signal_connect (
+    G_OBJECT (tw_prv->drag), "drag-begin",
+    G_CALLBACK (on_drag_begin), self);
+  g_signal_connect (
+    G_OBJECT (tw_prv->drag), "drag-update",
+    G_CALLBACK (on_drag_update), self);
+  g_signal_connect_after (
+    GTK_WIDGET (tw_prv->event_box),
+    "drag-begin",
+    G_CALLBACK(on_dnd_drag_begin), self);
+  g_signal_connect (
+    GTK_WIDGET (self), "drag-data-received",
+    G_CALLBACK(on_drag_data_received), self);
+  g_signal_connect (
+    GTK_WIDGET (tw_prv->event_box),
+    "drag-data-get",
+    G_CALLBACK (on_drag_data_get), self);
+  g_signal_connect (
+    GTK_WIDGET (self), "drag-motion",
+    G_CALLBACK (on_drag_motion), self);
+  g_signal_connect (
+    GTK_WIDGET (self), "drag-leave",
+    G_CALLBACK (on_drag_leave), self);
   g_signal_connect (
     G_OBJECT(self), "destroy",
     G_CALLBACK (on_destroy),  NULL);
@@ -796,8 +1164,12 @@ track_widget_class_init (TrackWidgetClass * _klass)
     klass,
     TrackWidget,
     event_box);
-  /*gtk_widget_class_bind_template_child_private (*/
-    /*klass,*/
-    /*TrackWidget,*/
-    /*lanes_box);*/
+  gtk_widget_class_bind_template_child_private (
+    klass,
+    TrackWidget,
+    highlight_top_box);
+  gtk_widget_class_bind_template_child_private (
+    klass,
+    TrackWidget,
+    highlight_bot_box);
 }
