@@ -77,11 +77,6 @@ handler (int sig) {
     }
 
   g_message ("%s", str);
-  tmp =
-    g_strdup_printf (
-      _("An error has occurred causing Zrythm to "
-        "crash. Please help us fix this error by "
-        "<a href=\"https://savannah.nongnu.org/support/?func=additem&group=zrythm\">submitting a bug report</a>, using the text below."));
 
   GtkDialogFlags flags =
     GTK_DIALOG_DESTROY_WITH_PARENT;
@@ -93,20 +88,46 @@ handler (int sig) {
       GTK_BUTTONS_CLOSE,
       NULL);
 
-  gtk_message_dialog_set_markup (
-    GTK_MESSAGE_DIALOG (dialog),
+  /* %23 is hash, %0A is new line */
+  char * ver = zrythm_get_version (0);
+  char * report_template =
+    g_strdup_printf (
+      "# Steps to reproduce\n"
+      "> Write a list of steps to reproduce the "
+      "bug\n\n"
+      "# What happens?\n"
+      "> Please tell us what happened\n\n"
+      "# What is expected?\n"
+      "> What is expected to happen?\n\n"
+      "# Version\n%s\n\n"
+      "# Other info\n"
+      "> Context, etc.\n\n"
+      "# Backtrace\n%s\n",
+      ver,
+      str
+      );
+  g_free (ver);
+  char * report_template_escaped =
+    g_uri_escape_string (
+      report_template, NULL, FALSE);
+  char * atag =
+    g_strdup_printf (
+      "<a href=\"https://savannah.nongnu.org/support/?func=additem&amp;group=zrythm&amp;prefill[summary]=Bug Report (enter details)&amp;prefill[details]=%s\">",
+      report_template_escaped);
+  char * markup =
+    g_strdup_printf (
       _("Zrythm has crashed. Please help us fix "
         "this by "
-        "<a href=\"https://savannah.nongnu.org/support/?func=additem&amp;group=zrythm\">submitting a bug report</a>. "
-        "Please include the following text, along "
-        "with steps to reproduce"));
+        "%ssubmitting a bug report%s."),
+      atag,
+      "</a>");
+
+  gtk_message_dialog_set_markup (
+    GTK_MESSAGE_DIALOG (dialog),
+    markup);
   gtk_message_dialog_format_secondary_markup (
     GTK_MESSAGE_DIALOG (dialog),
-    "%s\n%s\n\n%s\n%s",
-    "# Version",
-    PACKAGE_VERSION,
-    "# Backtrace",
-    str);
+    "%s", str);
   GtkLabel * label =
     z_gtk_message_dialog_get_label (
       GTK_MESSAGE_DIALOG (dialog), 1);
@@ -117,7 +138,10 @@ handler (int sig) {
 
   free (strings);
   g_free (str);
-  g_free (tmp);
+  g_free (report_template);
+  g_free (atag);
+  g_free (markup);
+  g_free (report_template_escaped);
 #endif
   exit(1);
 }
@@ -125,16 +149,18 @@ handler (int sig) {
 static void
 print_version ()
 {
+  char * ver = zrythm_get_version (0);
   fprintf (
     stdout,
     "Zrythm %s\nbuilt with %s %s\n"
     "%s\n%s\n%s\n%s\n",
-    PACKAGE_VERSION,
+    ver,
     COMPILER, COMPILER_VERSION,
     "Copyright (C) 2018-2019 Alexandros Theodotou",
     "License AGPLv3+: GNU AGPL version 3 or later <https://www.gnu.org/licenses/agpl.html>",
     "This is free software: you are free to change and redistribute it.",
     "There is NO WARRANTY, to the extent permitted by law.");
+  g_free (ver);
 }
 
 static void
@@ -198,16 +224,18 @@ main (int    argc,
         }
     }
 
+  char * ver = zrythm_get_version (0);
   fprintf (
     stdout,
-    _("Zrythm-%s Copyright (C) 2018-2019 Alexandros Theodotou et al.\n\n"
+    _("Zrythm-%s Copyright (C) 2018-2019 Alexandros Theodotou\n\n"
     "Zrythm comes with ABSOLUTELY NO WARRANTY!\n\n"
     "This is free software, and you are welcome to redistribute it\n"
     "under certain conditions. Look at the file `COPYING' within this\n"
     "distribution for details.\n\n"
     "Write comments and bugs to https://savannah.nongnu.org/support/?group=zrythm\n"
     "Support this project at https://liberapay.com/Zrythm\n\n"),
-    PACKAGE_VERSION);
+    ver);
+  g_free (ver);
 
   /* unset GTK_THEME */
   g_message ("GTK_THEME=%s", getenv ("GTK_THEME"));
