@@ -164,6 +164,7 @@ create_default (Project * self)
   TRACKLIST->master_track = track;
   tracklist_selections_add_track (
     TRACKLIST_SELECTIONS, track);
+  self->last_selection = SELECTION_TYPE_TRACK;
 
   /* create untitled project */
   char * untitled_project = _("Untitled Project");
@@ -392,7 +393,8 @@ project_load (char * filename)
     create_default (PROJECT);
 
   /* set the version */
-  PROJECT->version = PACKAGE_VERSION;
+  PROJECT->version =
+    zrythm_get_version (0);
 
   return 0;
 }
